const script = registerScript({
  name: "packet debugger",
  version: "1.0.0",
  authors: ["Trikaes"],
});

script.registerModule(
  {
    name: "packetdebugger",
    category: "Misc",
    description: "Prints packets",
    settings: {
      client: Setting.boolean({
        name: "C2S",
        default: true,
      }),
      server: Setting.boolean({
        name: "S2C",
        default: true,
      }),
      contents: Setting.boolean({
        name: "Show Contents",
        default: true,
      }),
      packet_filter: Setting.textArray({
        name: "Packet Filter (name)",
        default: ["CommonPingS2CPacket"],
      }),
    },
  },
  (mod) => {
    mod.on("packet", (event) => {
      const packet = event.packet;
      const packetString = packet.toString();

      if (
        shouldProcessPacket(packetString, mod.settings) &&
        !shouldIgnorePacket(packetString, mod.settings.packet_filter.value)
      ) {
        const message = constructPacketMessage(
          packet,
          mod.settings.contents.value
        );
        Client.displayChatMessage(`§7${message}`);
      }
    });
  }
);

function shouldProcessPacket(packetString, settings) {
  return (
    (settings.client.value && packetString.includes("C2S")) ||
    (settings.server.value && packetString.includes("S2C"))
  );
}

function shouldIgnorePacket(packetString, filters) {
  return filters.some((filter) => packetString.includes(filter));
}

function constructPacketMessage(packet, showContents) {
  const packetName = extractPacketName(packet.toString());
  let message = `§6Packet Name: §e${packetName}`;

  if (showContents) {
    const contents = extractPacketContents(packet);
    if (contents) {
      message += `\n§6Packet Contents:\n${contents}`;
    }
  }

  return message;
}

function extractPacketContents(packet) {
  return Object.entries(packet)
    .map(([key, value]) =>
      typeof value !== "function" ? `§6${key}: §e${value}` : null
    )
    .filter((item) => item !== null)
    .join("\n");
}

function extractPacketName(packetString) {
  const matches = packetString.match(/(\w+)Packet/);
  let packetName = matches ? matches[1] : packetString;
  return packetName.endsWith("Packet") ? packetName : packetName + "Packet";
}
